@@ -136,6 +136,7 @@ extern int h_errno;
 /* response time array flags */
 #define RESP_WAITING    -1
 #define RESP_UNUSED     -2
+#define RESP_ERROR      -3
 
 /* debugging flags */
 #if defined( DEBUG ) || defined( _DEBUG )
@@ -367,8 +368,9 @@ int main( int argc, char **argv )
 
     s = open_ping_socket();
 
-    if(uid = getuid()) {
-        seteuid( getuid() );
+    if((uid = getuid())) {
+        /* drop privileges */
+        setuid( getuid() );
     }
 
     prog = argv[0];
@@ -390,7 +392,8 @@ int main( int argc, char **argv )
             break;
         
         case 'r':
-            retry = ( unsigned int )atoi( optarg );
+            if (!sscanf(optarg,"%i",&retry))
+                usage(1);
             break;
         
         case 'i':
@@ -421,9 +424,7 @@ int main( int argc, char **argv )
             break;
 
         case 'b':
-            errno = 0;
-            ping_data_size = (unsigned int) strtol(optarg, (char **)NULL, 10);
-            if( errno )
+            if (!sscanf(optarg,"%i",&ping_data_size))
                 usage(1);
             
             break;
@@ -508,27 +509,8 @@ int main( int argc, char **argv )
             exit( 0 );
 
         case 'f': 
-#ifdef ENABLE_F_OPTION
             filename = optarg;
-            generate_flag = 0;
             break;
-#else
-            if( getuid() )
-            {
-                printf( "%s: this option can only be used by root.\n", argv[0] );
-                printf( "%s: fping will read from stdin by default.\n", argv[0] );
-                exit( 3 );
-
-            }/* IF */
-            else
-            {
-                filename = optarg;
-                generate_flag = 0;
-
-            }/* ELSE */
-
-            break;
-#endif /* ENABLE_F_OPTION */
 
         case 'g':
             /* use IP list generation */
@@ -567,9 +549,13 @@ int main( int argc, char **argv )
                     perror("setting type of service octet IP_TOS");
                 }
             }
+            else {
+                usage(1);
+            }
             break;
         default:
-            usage(1);
+            fprintf(stderr, "see 'fping -h' for usage information\n");
+            exit(1);
             break;
 
         }/* SWITCH */
@@ -578,21 +564,21 @@ int main( int argc, char **argv )
     /* validate various option settings */
 
     if (ttl > 255) {
-        fprintf(stderr, "ping: ttl %u out of range\n", ttl);  
-        usage(1);
+        fprintf(stderr, "ttl %u out of range\n", ttl);  
+        exit(1);
     }  
 
     if( unreachable_flag && alive_flag )
     {
         fprintf( stderr, "%s: specify only one of a, u\n", argv[0] );
-        usage(1);
+        exit(1);
 
     }/* IF */
 
     if( count_flag && loop_flag )
     {
         fprintf( stderr, "%s: specify only one of c, l\n", argv[0] );
-        usage(1);
+        exit(1);
     
     }/* IF */
     
@@ -605,15 +591,14 @@ int main( int argc, char **argv )
         fprintf( stderr, "%s: these options are too risky for mere mortals.\n", prog );
         fprintf( stderr, "%s: You need i >= %u, p >= %u, r < %u, and t >= %u\n",
             prog, MIN_INTERVAL, MIN_PERHOST_INTERVAL, MAX_RETRY, MIN_TIMEOUT );
-        usage(1);
-
+        exit(1);
     }/* IF */
     
     if( ( ping_data_size > MAX_PING_DATA ) || ( ping_data_size < MIN_PING_DATA ) )
     {
         fprintf( stderr, "%s: data size %u not valid, must be between %u and %u\n",
             prog, ping_data_size, (unsigned int) MIN_PING_DATA, (unsigned int) MAX_PING_DATA );
-        usage(1);
+        exit(1);
     
     }/* IF */
     
@@ -621,7 +606,7 @@ int main( int argc, char **argv )
     {
         fprintf( stderr, "%s: backoff factor %.1f not valid, must be between %.1f and %.1f\n",
             prog, backoff, MIN_BACKOFF_FACTOR, MAX_BACKOFF_FACTOR );
-        usage(1);
+        exit(1);
     
     }/* IF */
 
@@ -629,7 +614,7 @@ int main( int argc, char **argv )
     {
         fprintf( stderr, "%s: count %u not valid, must be less than %u\n",
             prog, count, MAX_COUNT );
-        usage(1);
+        exit(1);
     
     }/* IF */
     
@@ -793,7 +778,7 @@ int main( int argc, char **argv )
     }
     
     if( !num_hosts )
-        exit( 2 );
+        exit(1);
 
     if(src_addr_present) {
         socket_set_src_addr(s, src_addr);
@@ -879,10 +864,6 @@ void add_cidr(char *addr)
     *addr_end = '\0';
     mask_str = addr_end + 1;
     mask = atoi(mask_str);
-    if(mask < 1 || mask > 30) {
-        fprintf(stderr, "Error: netmask must be between 1 and 30 (is: %s)\n", mask_str);
-        exit(2);
-    }
 
     /* parse address (IPv4 only) */
     memset(&addr_hints, 0, sizeof(struct addrinfo));
@@ -891,14 +872,20 @@ void add_cidr(char *addr)
     ret = getaddrinfo(addr, NULL, &addr_hints, &addr_res);
     if(ret) {
         fprintf(stderr, "Error: can't parse address %s: %s\n", addr, gai_strerror(ret));
-        exit(2);
+        exit(1);
     }
     if(addr_res->ai_family != AF_INET) {
         fprintf(stderr, "Error: -g works only with IPv4 addresses\n");
-        exit(2);
+        exit(1);
     }
     net_addr = ntohl(((struct sockaddr_in *) addr_res->ai_addr)->sin_addr.s_addr);
 
+    /* check mask */
+    if(mask < 1 || mask > 30) {
+        fprintf(stderr, "Error: netmask must be between 1 and 30 (is: %s)\n", mask_str);
+        exit(1);
+    }
+
     /* convert mask integer from 1 to 32 to a bitmask */
     bitmask = ((unsigned long) 0xFFFFFFFF) << (32-mask);
 
@@ -933,11 +920,11 @@ void add_range(char *start, char *end)
     ret = getaddrinfo(start, NULL, &addr_hints, &addr_res);
     if(ret) {
         fprintf(stderr, "Error: can't parse address %s: %s\n", start, gai_strerror(ret));
-        exit(2);
+        exit(1);
     }
     if(addr_res->ai_family != AF_INET) {
         fprintf(stderr, "Error: -g works only with IPv4 addresses\n");
-        exit(2);
+        exit(1);
     }
     start_long = ntohl(((struct sockaddr_in *) addr_res->ai_addr)->sin_addr.s_addr);
 
@@ -948,11 +935,11 @@ void add_range(char *start, char *end)
     ret = getaddrinfo(end, NULL, &addr_hints, &addr_res);
     if(ret) {
         fprintf(stderr, "Error: can't parse address %s: %s\n", end, gai_strerror(ret));
-        exit(2);
+        exit(1);
     }
     if(addr_res->ai_family != AF_INET) {
         fprintf(stderr, "Error: -g works only with IPv4 addresses\n");
-        exit(2);
+        exit(1);
     }
     end_long = ntohl(((struct sockaddr_in *) addr_res->ai_addr)->sin_addr.s_addr);
 
@@ -991,7 +978,7 @@ void main_loop()
 
                 /* Send the ping */
 		/*printf("Sending ping after %d ms\n", lt/100); */
-                if(!send_ping(s, h)) goto wait_for_reply;
+                send_ping(s, h);
 
                 /* Check what needs to be done next */
                 if(!loop_flag && !count_flag) {
@@ -1410,7 +1397,8 @@ int send_ping( int s, HOST_ENTRY *h )
     char *buffer;
     FPING_ICMPHDR *icp;
     int n;
-        int myseq;
+    int myseq;
+    int ret = 1;
 
     buffer = ( char* )malloc( ( size_t )ping_pkt_size );
     if( !buffer )
@@ -1452,42 +1440,38 @@ int send_ping( int s, HOST_ENTRY *h )
         && errno != EHOSTDOWN
 #endif
     ) {
-        if( verbose_flag || unreachable_flag )
-        {
+        if( verbose_flag || unreachable_flag ) {
             printf( "%s", h->host );
             if( verbose_flag )
                 printf( " error while sending ping: %s\n", strerror( errno ) );
             
             printf( "\n" );
-
-        }/* IF */
+        }
         
-        h->num_sent++;
-        h->num_sent_i++;
-        h->waiting++;
-        num_pingsent++;
-        last_send_time = h->last_send_time;
-        free( buffer );
-        return(1);
-    }
+        if( !loop_flag )
+            h->resp_times[h->num_sent] = RESP_ERROR;
 
-    /* mark this trial as outstanding */
-    if( !loop_flag )
-        h->resp_times[h->num_sent] = RESP_WAITING;
+        ret = 0;
+    }
+    else {
+        /* mark this trial as outstanding */
+        if( !loop_flag )
+            h->resp_times[h->num_sent] = RESP_WAITING;
 
 #if defined( DEBUG ) || defined( _DEBUG )
-    if( sent_times_flag )
-        h->sent_times[h->num_sent] = timeval_diff( &h->last_send_time, &start_time );
-#endif /* DEBUG || _DEBUG */
+        if( sent_times_flag )
+            h->sent_times[h->num_sent] = timeval_diff( &h->last_send_time, &start_time );
+#endif
+    }
 
     h->num_sent++;
     h->num_sent_i++;
     h->waiting++;
     num_pingsent++;
     last_send_time = h->last_send_time;
-    
     free( buffer );
-    return(1);
+
+    return(ret);
 }
 
 
@@ -1637,7 +1621,7 @@ int wait_for_reply(long wait_time)
     {
         if( ( this_count >= 0 ) && ( this_count < trials ) )
         {
-            if( h->resp_times[this_count] != RESP_WAITING )
+            if( h->resp_times[this_count] >= 0 )
             {
                 if( !per_recv_flag )
                 {
@@ -2002,45 +1986,10 @@ void add_name( char *name )
     host_ent = gethostbyname( name ); 
     if( host_ent == NULL )
     { 
-        if( h_errno == TRY_AGAIN )
-        { 
-            u_sleep( DNS_TIMEOUT ); 
-            host_ent = gethostbyname( name );
-
-        }/* IF */
-
-        if( host_ent == NULL )
-        {
-#ifdef NIS_GROUPS
-
-            /* maybe it's the name of a NIS netgroup */
-            char *machine, *user_ignored, *domain_ignored;
-            setnetgrent( name );
-            if( getnetgrent( &machine, &user_ignored, &domain_ignored ) == 0 )
-            {
-                endnetgrent();
-                print_warning("%s address not found\n", name );
-                
-                num_noaddress++;
-                return;
-            
-            }/* IF */
-            else
-                add_name(machine);
-
-            while( getnetgrent( &machine, &user_ignored, &domain_ignored ) )
-                add_name(machine);
-      
-            endnetgrent();
-            return;
-#else
             print_warning("%s address not found\n", name );
-            
             num_noaddress++;
             return ; 
-#endif /* NIS_GROUPS */
-        }/* IF */
-    }/* IF */
+    }
 
     if(host_ent->h_addrtype != AF_INET) {
         print_warning("%s: IPv6 address returned by gethostbyname (options inet6 in resolv.conf?)\n", name );
@@ -2705,23 +2654,23 @@ void usage(int is_error)
     fprintf(out, "                (ex. %s -g 192.168.1.0 192.168.1.255 or %s -g 192.168.1.0/24)\n", prog, prog );
     fprintf(out, "   -H n       Set the IP TTL value (Time To Live hops)\n");
     fprintf(out, "   -i n       interval between sending ping packets (in millisec) (default %d)\n", interval / 100 );
+#ifdef SO_BINDTODEVICE
+        fprintf(out, "   -I if      bind to a particular interface\n");
+#endif
     fprintf(out, "   -l         loop sending pings forever\n" );
     fprintf(out, "   -m         ping multiple interfaces on target host\n" );
     fprintf(out, "   -n         show targets by name (-d is equivalent)\n" );
+    fprintf(out, "   -O n       set the type of service (tos) flag on the ICMP packets\n" );
     fprintf(out, "   -p n       interval between ping packets to one target (in millisec)\n" );
     fprintf(out, "                (in looping and counting modes, default %d)\n", perhost_interval / 100 );
     fprintf(out, "   -q         quiet (don't show per-target/per-ping results)\n" );
     fprintf(out, "   -Q n       same as -q, but show summary every n seconds\n" );
     fprintf(out, "   -r n       number of retries (default %d)\n", DEFAULT_RETRY );
     fprintf(out, "   -s         print final stats\n" );
-#ifdef SO_BINDTODEVICE
-        fprintf(out, "   -I if      bind to a particular interface\n");
-#endif
     fprintf(out, "   -S addr    set source address\n" );
     fprintf(out, "   -t n       individual target initial timeout (in millisec) (default %d)\n", timeout / 100 );
     fprintf(out, "   -T n       ignored (for compatibility with fping 2.4)\n");
     fprintf(out, "   -u         show targets that are unreachable\n" );
-    fprintf(out, "   -O n       set the type of service (tos) flag on the ICMP packets\n" );
     fprintf(out, "   -v         show version\n" );
     fprintf(out, "   targets    list of targets to check (if no -f specified)\n" );
     fprintf(out, "\n");
